import logging
import os
import sys

import aiosqlite as sqlite
from disnake import Guild, Intents, Message
from disnake.ext import commands
from disnake.ext.commands import Bot
from dotenv import find_dotenv, load_dotenv

# Logger setup
logger_disnake = logging.getLogger("disnake")
logger_disnake.setLevel(logging.INFO)

log = logging.getLogger("nomen")
log.setLevel(logging.DEBUG)

formatter = logging.Formatter("%(asctime)s:%(levelname)s:%(name)s: %(message)s")
handler = logging.StreamHandler(sys.stdout)
handler.setFormatter(formatter)
logger_disnake.addHandler(handler)
log.addHandler(handler)

if load_dotenv(find_dotenv(usecwd=True)):
    log.info("Loaded .env")
else:
    log.debug("Didn't find .env")

TOKEN = os.getenv("TOKEN")
DEFAULT_PREFIX = os.getenv("DEFAULT_PREFIX") or ">"


async def get_prefix(the_bot, message: Message):
    if not message.guild:
        return commands.when_mentioned_or(DEFAULT_PREFIX)(the_bot, message)
    gp = await the_bot.get_guild_prefix(message.guild)
    return commands.when_mentioned_or(gp)(the_bot, message)


class Nomen(Bot):
    def __init__(self, prefix, description=None, **options):
        super().__init__(
            prefix,
            description=description,
            command_sync_flags=options.get("sync_flags"),
            allowed_mentions=options.get("allowed_mentions"),
            intents=options.get("intents"),
        )

        # self.db = self.loop.run_until_complete(sqlite.connect("nomen.db"))
        self.prefixes = {}

    async def get_guild_prefix(self, guild: Guild):
        if guild.id in self.prefixes:
            return self.prefixes[guild.id]
        # FIXME: Fetch with SQLite

        # self.prefixes[guild.id] = prefix
        return DEFAULT_PREFIX

    async def set_guild_prefix(self, guild: Guild, prefix):
        # FIXME: Fetch with SQLite
        self.prefixes[guild.id] = prefix

    async def close(self):
        await super().close()
        await self.db.close()


bot = Nomen(
    prefix=get_prefix,
    description="Keeper of Names",
    intents=Intents(
        guilds=True,
        messages=True,
        message_content=True,
    ),
    sync_flags=commands.CommandSyncFlags(
        sync_commands=False,
    ),
)


@bot.event
async def on_ready():
    log.info("Logged in as:")
    log.info(bot.user)
    log.info(bot.user.id)
    log.info("-------------")


@bot.event
async def on_command_error(ctx, error):
    if isinstance(error, commands.CommandNotFound):
        return


async def handle_triggers(ctx, message):
    log.debug(f"Handling triggers for message '{ctx.message.content}'")


@bot.event
async def on_message(message):
    if message.author == bot.user:
        return

    ctx = await bot.get_context(message)
    if ctx.valid:
        await bot.invoke(ctx)
    else:
        await handle_triggers(ctx, message)


@bot.command()
@commands.check(commands.is_owner())
async def echo(ctx, arg):
    await ctx.send(arg)


@bot.command()
async def ping(ctx):
    await ctx.send("Pong")


@bot.command()
@commands.check(commands.guild_only())
async def prefix(ctx, prefix=None):
    if not prefix:
        prefix = await ctx.bot.get_guild_prefix(ctx.guild)
        await ctx.send(f"{ctx.guild.name} prefix is `{prefix}`")
    else:
        await ctx.bot.set_guild_prefix(ctx.guild, prefix)
        await ctx.send(f"Set {ctx.guild.name} prefix to `{prefix}`")


def run():
    bot.run(TOKEN)
